@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Trax Authors.
+# Copyright 2021 The Trax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 
 # Lint as: python3
-"""Layers defined in trax."""
+"""Layers: trainable functions as neural network building blocks."""
 
 import gin
 # We create a flat layers.* namespace for uniform calling conventions as we
@@ -28,12 +28,14 @@ from trax.layers.base import *
 from trax.layers.combinators import *
 from trax.layers.convolution import *
 from trax.layers.core import *
+from trax.layers.deconvolution import *
 from trax.layers.initializers import *
 from trax.layers.metrics import *
 from trax.layers.normalization import *
 from trax.layers.pooling import *
 from trax.layers.research.efficient_attention import *
 from trax.layers.research.position_encodings import *
+from trax.layers.research.rel_attention import *
 from trax.layers.research.sparsity import *
 from trax.layers.reversible import *
 from trax.layers.rnn import *
@@ -66,19 +68,47 @@ LayerNorm = layer_configure(LayerNorm)
 FilterResponseNorm = layer_configure(FilterResponseNorm)
 ThresholdedLinearUnit = layer_configure(ThresholdedLinearUnit)
 
-CausalAttention = layer_configure(CausalAttention, blacklist=['mode'])
-CausalFavor = layer_configure(CausalFavor, blacklist=['mode'])
+Attention = layer_configure(Attention, denylist=['mode'])
+CausalAttention = layer_configure(CausalAttention, denylist=['mode'])
+FavorAttention = layer_configure(FavorAttention, denylist=['mode'])
+Favor = layer_configure(Favor, denylist=['mode'])
+CausalFavor = layer_configure(CausalFavor, denylist=['mode'])
+CausalFavorAttention = layer_configure(CausalFavorAttention, denylist=['mode'])
 DotProductCausalAttention = layer_configure(
-    DotProductCausalAttention, blacklist=['mode'])
-SelfAttention = layer_configure(SelfAttention, blacklist=['mode'])
+    DotProductCausalAttention, denylist=['mode'])
+SelfAttention = layer_configure(SelfAttention, denylist=['mode'])
 ModularCausalAttention = layer_configure(ModularCausalAttention,
-                                         blacklist=['mode'])
-LSHSelfAttention = layer_configure(LSHSelfAttention, blacklist=['mode'])
-EncDecAttention = layer_configure(EncDecAttention, blacklist=['mode'])
+                                         denylist=['mode'])
+LowRankCausalAttention = layer_configure(LowRankCausalAttention,
+                                         denylist=['mode'])
+MultiplicativeCausalAttention = layer_configure(MultiplicativeCausalAttention,
+                                                denylist=['mode'])
+MultiplicativeModularCausalAttention = layer_configure(
+    MultiplicativeModularCausalAttention, denylist=['mode'])
+ConvCausalAttention = layer_configure(ConvCausalAttention, denylist=['mode'])
+MultiplicativeConvCausalAttention = layer_configure(
+    MultiplicativeConvCausalAttention, denylist=['mode'])
+ConvTranspose = layer_configure(ConvTranspose)
+LSHSelfAttention = layer_configure(LSHSelfAttention, denylist=['mode'])
+PureLSHSelfAttention = layer_configure(PureLSHSelfAttention, denylist=['mode'])
+MixedLSHSelfAttention = layer_configure(
+    MixedLSHSelfAttention, denylist=['mode'])
+PureLSHSelfAttentionWrapper = layer_configure(
+    PureLSHSelfAttentionWrapper, denylist=['mode'])
+EncDecAttention = layer_configure(EncDecAttention, denylist=['mode'])
 
 InfinitePositionalEncoding = layer_configure(
-    InfinitePositionalEncoding, blacklist=['mode'])
+    InfinitePositionalEncoding, denylist=['mode'])
 TimeBinPositionalEncoding = layer_configure(
-    TimeBinPositionalEncoding, blacklist=['mode'])
+    TimeBinPositionalEncoding, denylist=['mode'])
 
 AtariConvInit = layer_configure(AtariConvInit)
+CrossEntropyLossWithLogSoftmax = layer_configure(CrossEntropyLossWithLogSoftmax)
+WeightedCategoryAccuracy = layer_configure(WeightedCategoryAccuracy)
+SequenceAccuracy = layer_configure(SequenceAccuracy)
+CategoryCrossEntropy = layer_configure(CategoryCrossEntropy)
+WeightedCategoryCrossEntropy = layer_configure(WeightedCategoryCrossEntropy)
+MacroAveragedFScore = layer_configure(MacroAveragedFScore)
+WeightedFScore = layer_configure(WeightedFScore)
+RelativeAttentionLayer = layer_configure(RelativeAttentionLayer)
+RelativeAttentionLMLayer = layer_configure(RelativeAttentionLMLayer)
