@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Trax Authors.
+# Copyright 2021 The Trax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,10 @@
 # limitations under the License.
 
 # Lint as: python3
-"""Accelerate layer that allows for fast computation on accelerators."""
+"""Modifications to data and computation to use accelerators (better)."""
 
 import jax
+import numpy as np
 from trax import fastmath
 from trax.fastmath import numpy as jnp
 from trax.layers import base
@@ -35,27 +36,26 @@ class Accelerate(base.Layer):
   then 0-padding is added to make it divisible and the output may be affected
   if it relies on layers like batch normalization.
 
-  This layer does not require calling `init` if the underlying layer has
-  already been initialized, so it can be used as follows:
-  ```
-  layer = tl.Serial(...)
-  layer.init(...)
-  fast_layer = tl.Accelerate(layer)
-  y = fast_layer(x)  # Computes y splitting x on batch and running data-parallel
-  ```
+  This layer does not require calling ``init`` if the underlying layer has
+  already been initialized, so it can be used as follows::
+
+      layer = tl.Serial(...)
+      layer.init(...)
+      fast_layer = tl.Accelerate(layer)
+      y = fast_layer(x)  # Split x on batch and run data-parallel
 
   In case the weights of this layer need to be set using the weights of
-  the sublayer, use the `replicate_weights` function:
-  ```
-  # Instead of layer.weights = new_weights:
-  fast_layer.replicate_weights(new_weights)
-  ```
+  the sublayer, use the ``replicate_weights`` function::
+
+      # Instead of layer.weights = new_weights:
+      fast_layer.replicate_weights(new_weights)
+
   """
 
   def __init__(self, layer, n_devices=None):
     super().__init__(n_in=layer.n_in, n_out=layer.n_out)
     self._sublayers = [layer]
-    self._n_devices = n_devices or fastmath.device_count()
+    self._n_devices = n_devices or fastmath.local_device_count()
     self._jit_pure_fn = jit_forward(
         layer.pure_fn, self._n_devices, do_mean=False)
 
@@ -65,7 +65,7 @@ class Accelerate(base.Layer):
     return self._sublayers[0]
 
   def pure_fn(self, x, weights, state, rng, use_cache=False):
-    """Calls self.sublayer.pure_fn in an accelerated way."""
+    """Calls ``self.sublayer.pure_fn`` in an accelerated way."""
     # Check if we can divide x evenly across devices.
     # Note: x can be a list/tuple because the underlying layer may take
     # its input as a list/tuple, ex: (inputs, targets, weight).
@@ -91,16 +91,27 @@ class Accelerate(base.Layer):
       y = padded_y[:x.shape[0]]
     return y, state
 
+  def _prepare_weights(self, weights):
+    """Replicate or shard weights for the number of devices requested."""
+    if base.N_WEIGHTS_SHARDS > 1:
+      if base.N_WEIGHTS_SHARDS % self._n_devices != 0:
+        raise ValueError(f'Number of shards ({base.N_WEIGHTS_SHARDS}) must '
+                         f'be a multiple of n_devices ({self._n_devices}).')
+      return base.shard(weights, base.N_WEIGHTS_SHARDS)
+    else:
+      return for_n_devices(weights, self._n_devices)
+
   def init(self, input_signature):
-    """Calls self.sublayer.init and replicated on multiple devices."""
+    """Calls ``self.sublayer.init`` and replicates its values onto devices."""
     weights, state = self.sublayer.init(input_signature, use_cache=True)
-    self._weights = for_n_devices(weights, self._n_devices)
+    self._weights = self._prepare_weights(weights)
     self._state = for_n_devices(state, self._n_devices)
+    return (self.weights, self.state)
 
   def replicate_weights(self, weights):
     """Sets the weights of the sublayer and replicates them for this layer."""
     self.sublayer.weights = weights
-    self._weights = for_n_devices(weights, self._n_devices)
+    self._weights = self._prepare_weights(weights)
 
   def replicate_state(self, state):
     """Sets the state of the sublayer and replicates it for this layer."""
@@ -108,7 +119,7 @@ class Accelerate(base.Layer):
     self._state = for_n_devices(state, self._n_devices)
 
   def _unreplicate(self, x):
-    """Return a single-device version of x using the first component only."""
+    """Returns a single-device version of ``x``."""
     if self._n_devices < 2:
       return x
     return fastmath.nested_map(lambda y: y[0], x)
@@ -117,7 +128,7 @@ class Accelerate(base.Layer):
   def weights(self):
     # Override the getter so it works even if only sublayer is initialized.
     if self._weights is base.EMPTY_WEIGHTS:
-      self._weights = for_n_devices(self.sublayer.weights, self._n_devices)
+      self._weights = self._prepare_weights(self.sublayer.weights)
     return self._weights
 
   @weights.setter
@@ -138,18 +149,14 @@ class Accelerate(base.Layer):
     self.sublayer.state = self._unreplicate(state)
 
 
+# TODO(jonni): Rename, since implementation does not use pmean.
 def mean_or_pmean(n_devices, x, axis=None):
-  """jnp.mean or pmean.
-
-  `x` is a distributed value. Directly calling jnp.mean on `x` means stacking
-  x's components together to form a large array and then doing jnp.mean on
-  it. In TF, stacking `x` will introduce D2H copy, so we use a collective
-  (pmean) here instead of directly calling jnp.mean for TF.
+  """Computes the mean of a distributed value ``x``.
 
   Args:
-    n_devices: number of devices.
-    x: a distributed array.
-    axis: the axis to reduce. Can only be 0 or None.
+    n_devices: Number of devices.
+    x: Distributed array.
+    axis: Axis along which to compute means; can only be ``0`` or ``None``.
 
   Returns:
     A local array.
@@ -166,7 +173,7 @@ def mean_or_pmean(n_devices, x, axis=None):
 
 
 def jit_forward(forward, n_devices, do_mean=True):
-  """Returns a JIT-compiled forward function running on `n_devices`."""
+  """Returns a JIT-compiled forward function running on ``n_devices``."""
   model_predict = _accelerate(forward, n_devices)
   # n_devices == 0 => CPU
   if n_devices < 2:
@@ -200,7 +207,7 @@ def _combine_devices(x_tuple):
 
 
 def _accelerate(f, n_devices):
-  """JIT-compiled version of `f` running on `n_devices`."""
+  """Returns an accelerated version of ``f`` running on ``n_devices``."""
   if n_devices == 0:  # no accelerators - run on CPU
     return fastmath.jit(f, device=jax.devices('cpu')[0])
 
@@ -210,8 +217,8 @@ def _accelerate(f, n_devices):
   return fastmath.pmap(f, axis_name='batch')
 
 
-def reshape_by_device(x, n_devices):
-  """Reshapes possibly nested `x` into a shape `(n_devices, ...)`."""
+def reshape_by_device(x, n_devices, pure_np=False):
+  """Reshapes possibly nested ``x`` into a shape ``(n_devices, ...)``."""
   def f(x):
     x_shape = list(x.shape)
     batch_size = x_shape[0]
@@ -220,15 +227,18 @@ def reshape_by_device(x, n_devices):
       raise ValueError(f'Number of devices ({n_devices}) does not evenly '
                        f'divide batch size ({batch_size}).')
     new_shape_prefix = [n_devices, batch_size_per_device]
-    return jnp.reshape(x, new_shape_prefix + x_shape[1:])
+    if pure_np:
+      return np.reshape(x, new_shape_prefix + x_shape[1:])
+    else:
+      return jnp.reshape(x, new_shape_prefix + x_shape[1:])
   return fastmath.nested_map(f, x)
 
 
 def for_n_devices(x, n_devices):
-  """Replicates/broadcasts `x` for `n_devices`."""
+  """Replicates/broadcasts ``x`` for ``n_devices``."""
   def f(x):
     if n_devices > 1 and fastmath.is_backend(fastmath.Backend.JAX):
-      return _multi_device_put(x)
+      return jax.device_put_replicated(x, jax.local_devices())
     elif n_devices > 1:
       return jnp.broadcast_to(x, (n_devices,) + jnp.asarray(x).shape)
     else:
@@ -237,12 +247,15 @@ def for_n_devices(x, n_devices):
 
 
 def on_cpu(x):
-  """Put the tensor x in CPU memory in JAX."""
-  return jax.device_put(x, jax.devices('cpu')[0])
+  """Puts ``x`` in CPU memory in JAX."""
+  if fastmath.is_backend(fastmath.Backend.JAX):
+    return jax.device_put(x, jax.devices('cpu')[0])
+  else:
+    return x
 
 
 def on_accelerator(x):
-  """Put the tensor x in (single) accelerator memory in JAX."""
+  """Puts ``x`` in (single) accelerator memory in JAX."""
   try:
     accelerator_devices = jax.devices('gpu')
   except RuntimeError:
@@ -252,35 +265,6 @@ def on_accelerator(x):
       accelerator_devices = []
   if not accelerator_devices:
     return x
-  assert len(accelerator_devices) == 1
+  if len(accelerator_devices) != 1:
+    return x
   return jax.device_put(x, accelerator_devices[0])
-
-
-def _multi_device_put(x, devices=None):
-  """Memory efficient multi-device replication / broadcast in JAX.
-
-  JAX uses a ShardedDeviceArray class that holds a list of device buffers
-  on separate devices for use with pmap'd computations.  Sharded arrays
-  are explicitly used to eliminate unnecessary inter-device transfer of
-  memory buffers between use in pmap'd computations.  The JAX API currently
-  does not have a multi-device 'put' function that copies a buffer onto
-  N devices in a memory-efficient fashion, so we implement our own here.
-
-  Args:
-    x: jax DeviceArray or numpy ndarray to be replicated.
-    devices: a jax.devices() list or subset thereof of devices to
-      replicate onto.  Should match the list passed to any pmaps
-      ingesting the replicated array.
-
-  Returns:
-    A ShardedDeviceArray with
-    dtype = x.dtype and shape = (n_devices,) + x.shape
-    that's backed by replicated device_buffers on each local device.
-  """
-  # Convert _FilledConstants that don't have device_buffer, etc.
-  if type(x) != jax.xla.DeviceArray:  # pylint: disable=unidiomatic-typecheck
-    x = jnp.array(x)
-  # Calculate the abstract shape of the replicated array.
-  if not devices:
-    devices = jax.local_devices()
-  return jax.api.device_put_sharded(len(devices) * [x], devices)
