@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Trax Authors.
+# Copyright 2021 The Trax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,12 +16,13 @@
 # Lint as: python3
 """Tests for trax.layers.attention."""
 
+import functools
 from absl.testing import absltest
 import numpy as np
 
 from trax import shapes
-
 import trax.layers as tl
+from trax.layers import test_utils
 
 
 class AttentionTest(absltest.TestCase):
@@ -111,6 +112,63 @@ class CausalAttentionTest(absltest.TestCase):
     y = layer(x)
     self.assertEqual(y.shape, (1, 3, 4))
 
+  def test_deterministic_eval(self):
+    d_model = 32
+    seq_len = 3
+    x_shape = (1, seq_len, d_model)
+    inp = np.ones(x_shape).astype(np.float32)
+
+    model_fn = functools.partial(
+        tl.CausalAttention,
+        d_feature=d_model,
+        n_heads=4,
+        )
+
+    test_utils.test_eval_is_deterministic(inp, model_fn)
+
+  def test_predict_equals_eval(self):
+    d_model = 32
+    seq_len = 10
+    x_shape = (1, seq_len, d_model)
+    inp = np.ones(x_shape).astype(np.float32)
+
+    model_fn = functools.partial(
+        tl.CausalAttention,
+        d_feature=d_model,
+        n_heads=4,
+        )
+
+    test_utils.test_eval_equals_predict(inp, model_fn)
+
+
+class PositionalEncodingTest(absltest.TestCase):
+
+  def test_simple_call(self):
+    layer = tl.PositionalEncoding(max_len=8)
+    x = np.array([[[2.0, 3.0, 4.0, 5.0], [1.0, 2.0, 3.0, 4.0]]])
+    layer.init(shapes.signature(x))
+    y = layer(x)
+    self.assertEqual(y.shape, (1, 2, 4))
+
+  def test_predict(self):
+    layer = tl.PositionalEncoding(max_len=8)
+    x = np.array([[[2.0, 3.0], [1.0, 2.0], [0.0, 1.0], [3.0, 4.0]]])
+    self.assertEqual(x.shape, (1, 4, 2))
+    layer.init(shapes.signature(x))
+    y = layer(x)
+    self.assertEqual(y.shape, (1, 4, 2))
+    layer = tl.PositionalEncoding(max_len=8, mode='predict')
+    layer.init(shapes.signature(x[:, :1, :]))
+    y0 = layer(x[:, :1, :])   # just the first token
+    self.assertEqual(y0.shape, (1, 1, 2))
+    self.assertTrue(np.array_equal(y0, y[:, :1, :]))
+    y1 = layer(x[:, 1:3, :])  # now the next 2 tokens
+    self.assertEqual(y1.shape, (1, 2, 2))
+    self.assertTrue(np.array_equal(y1, y[:, 1:3, :]))
+    y2 = layer(x[:, 3:4, :])  # final one token
+    self.assertEqual(y2.shape, (1, 1, 2))
+    self.assertTrue(np.array_equal(y2, y[:, 3:4, :]))
+
 
 if __name__ == '__main__':
   absltest.main()
