@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Trax Authors.
+# Copyright 2021 The Trax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -519,8 +519,11 @@ class SerialWithSideOutputsTest(absltest.TestCase):
     self.assertEqual(output_shapes, [(3,), (5,), (2,)])
 
 
+BACKENDS = [fastmath.Backend.JAX]
+
+
 @parameterized.named_parameters(
-    ('_' + b.value, b) for b in (fastmath.Backend.JAX, fastmath.Backend.TFNP))
+    ('_' + b.value, b) for b in BACKENDS)
 class ScanTest(parameterized.TestCase):
 
   def _AddWithCarry(self):  # pylint: disable=invalid-name
@@ -570,6 +573,22 @@ class ScanTest(parameterized.TestCase):
                          7600]
                        ])
 
+  def test_predict(self, backend):
+    with fastmath.use_backend(backend):
+      layer = tl.Scan(self._AddWithCarry(), axis=1, mode='predict')
+      xs = [np.array([[0, 1, 2]]),
+            np.array([90])]
+      ys = layer(xs)
+      self.assertEqual(as_list(ys),
+                       [[[90, 91, 93]],
+                        [93]])
+      xs = [np.array([[3, 4]]),
+            np.array([90])]
+      ys = layer(xs)
+      self.assertEqual(as_list(ys),
+                       [[[96, 100]],
+                        [100]])
+
   def test_multi_input(self, backend):
     def _MultiInputFn():  # pylint: disable=invalid-name
       def f(a, b, carry):
