@@ -19,7 +19,6 @@
 from trax import layers as tl
 from trax.fastmath import numpy as jnp
 from trax.models.research import configurable_transformer as ct
-from trax.models.research import transformer2 as t2
 
 
 # Layers are always CamelCase, but functions in general are snake_case
@@ -61,28 +60,35 @@ def DecoderBlock(d_model, d_ff, d_attention_key, d_attention_value,
     the layer.
   """
   # pylint: disable=g-complex-comprehension
-  attention_half_residuals = [
-      [tl.ReversibleHalfResidual(
-          tl.LayerNorm(center=center_layernorm),
-          attention_layer=ct.ApplyAttentionLayer(
-              attention_type, d_model, n_heads, d_attention_key,
-              d_attention_value, True, False, dropout, dropout,
-              attention_chunk_size, mode),
-          name='ReversibleHalfResidualDecoderAttn'),
-       tl.ReversibleSwap()
-      ] for _ in range(n_attention_layers)]
+  def _Attn():
+    return ct.ApplyAttentionLayer(
+        attention_type, d_model, n_heads, d_attention_key,
+        d_attention_value, True, False, dropout, dropout,
+        attention_chunk_size, mode)
 
-  feed_forwards = [
-      [tl.ReversibleHalfResidual(
-          ct.FeedForwardWithOptions(
-              d_model, d_ff, dropout, [-2], ff_activation, ff_dropout,
-              ff_chunk_size, ff_use_sru, ff_sparsity, center_layernorm,
-              mode, use_bfloat16),
-          name='ReversibleHalfResidualDecoderFF'),
-       tl.ReversibleSwap()
-      ] for _ in range(n_feedforward_layers)]
-  # pylint: enable=g-complex-comprehension
-  return attention_half_residuals + feed_forwards
+  def _FF():
+    return ct.FeedForwardWithOptions(
+        d_model, d_ff, dropout, [-2], ff_activation, ff_dropout,
+        ff_chunk_size, ff_use_sru, ff_sparsity, center_layernorm,
+        mode, use_bfloat16)
+
+  def _attention_half_residual():
+    return [
+        tl.ReversibleHalfResidual(tl.LayerNorm(center=center_layernorm),
+                                  attention_layer=_Attn(),
+                                  name='ReversibleHalfResidualDecoderAttn'),
+        tl.ReversibleSwap()
+    ]
+
+  def _feed_forward():
+    return [
+        tl.ReversibleHalfResidual(_FF(),
+                                  name='ReversibleHalfResidualDecoderFF'),
+        tl.ReversibleSwap()
+    ]
+
+  return ([_attention_half_residual() for _ in range(n_attention_layers)]
+          + [_feed_forward() for _ in range(n_feedforward_layers)])
 
 
 def ReformerLM(vocab_size,
@@ -361,43 +367,48 @@ def EncoderBlock(d_model, d_ff, n_heads, attention_type, dropout, ff_activation,
     # to 'eval' mode instead.
     mode = 'eval'
 
-  attention = ct.ApplyAttentionLayer(
-      attention_type=attention_type, d_model=d_model, n_heads=n_heads,
-      d_qk=d_model//n_heads, d_v=d_model//n_heads, masked=True, causal=False,
-      attention_dropout=dropout, output_dropout=dropout,
-      attention_chunk_size=attention_chunk_size, mode=mode)
+  def _Attn():
+    return ct.ApplyAttentionLayer(
+        attention_type=attention_type, d_model=d_model, n_heads=n_heads,
+        d_qk=d_model//n_heads, d_v=d_model//n_heads, masked=True, causal=False,
+        attention_dropout=dropout, output_dropout=dropout,
+        attention_chunk_size=attention_chunk_size, mode=mode)
+
+  def _FF():
+    return ct.FeedForwardWithOptions(
+        d_model, d_ff, dropout, [-2], ff_activation, ff_dropout,
+        ff_chunk_size, ff_use_sru, ff_sparsity, center_layernorm,
+        mode, use_bfloat16)
+
   # TODO(lukaszkaiser): refactor efficient attention layers to unify the API
   # If we're using standard attention, we need to pass reshaped mask and not
   # return the mask to be compatible with the EfficientAttention API.
+  attention = _Attn()
   if attention.n_out == 2:
-    def reshape_mask(mask):
-      return jnp.reshape(mask, (mask.shape[0], 1, 1, mask.shape[1]))
     attention = tl.Serial(
-        tl.Fn('ReshapeMask', lambda x, y: (x, reshape_mask(y)), n_out=2),
+        tl.Parallel([], _InsertAxes12()),
         attention,
         tl.Select([0], n_in=2)
     )
 
-  attention_half_residual = tl.ReversibleHalfResidual(
-      tl.LayerNorm(center=center_layernorm),
-      attention_layer=attention,
-      name='ReversibleHalfResidualEncoderAttn'
-  )
+  def _attention_half_residual():
+    return [
+        tl.ReversibleHalfResidual(tl.LayerNorm(center=center_layernorm),
+                                  attention_layer=attention,
+                                  name='ReversibleHalfResidualEncoderAttn'),
+        tl.ReversibleSwap()
+    ]
 
-  feed_forward = ct.FeedForwardWithOptions(
-      d_model, d_ff, dropout, [-2], ff_activation, ff_dropout,
-      ff_chunk_size, ff_use_sru, ff_sparsity, center_layernorm,
-      mode, use_bfloat16)
+  def _feed_forward():
+    layers = [
+        tl.ReversibleHalfResidual(_FF(),
+                                  name='ReversibleHalfResidualEncoderFF')
+    ]
+    if use_two_swaps_per_block:
+      layers.append(tl.ReversibleSwap())
+    return layers
 
-  encoder_block = [
-      attention_half_residual,
-      tl.ReversibleSwap(),
-      tl.ReversibleHalfResidual(feed_forward,
-                                name='ReversibleHalfResidualEncoderFF'),
-  ]
-  if use_two_swaps_per_block:
-    encoder_block.append(tl.ReversibleSwap())
-  return encoder_block
+  return _attention_half_residual() + _feed_forward()
 
 
 def EncoderDecoderBlock(d_model, d_ff, n_heads, dropout, ff_activation,
@@ -533,7 +544,7 @@ def Reformer(input_vocab_size,
       in_encoder,
       tl.Dup(),
       tl.ReversibleSerial(encoder_blocks),
-      tl.Fn('XYAvg', lambda x, y: (x + y) / 2.0),
+      _XYAvg(),
       tl.LayerNorm(),
   ])
   if mode == 'predict':
@@ -554,9 +565,7 @@ def Reformer(input_vocab_size,
       # Copy decoder tokens for use in loss.
       tl.Select([0, 1, 1]),                 # tok_e tok_d tok_d
       tl.Branch([], [tl.PaddingMask(),
-                     tl.Fn('Squeeze',
-                           lambda x: jnp.squeeze(x, (1, 2)), n_out=1)]),
-      #                                     # tok_e mask  tok_d .....
+                     _RemoveAxes12()]),     # tok_e mask  tok_d .....
 
       # Encode.
       encoder,                              # vec_e  mask tok_d .....
@@ -567,8 +576,7 @@ def Reformer(input_vocab_size,
       out_encoder,                          # vec_d vec_e mask .....
       tl.Dup(),                             # vec_d1 vec_d2 vec_e mask .....
       tl.ReversibleSerial(encoder_decoder_blocks),
-      tl.Fn('XYAvg',
-            lambda x, y: (x + y) / 2.0),    # vec_d vec_e mask .....
+      _XYAvg(),                             # vec_d vec_e mask .....
       tl.LayerNorm(),                       # vec_d vec_e mask .....
 
       # Map to output vocab.
@@ -577,281 +585,25 @@ def Reformer(input_vocab_size,
   )
 
 
-def Reformer2(input_vocab_size,
-              output_vocab_size=None,
-              d_model=512,
-              d_ff=2048,
-              d_attention_key=None,
-              d_attention_value=None,
-              n_encoder_layers=6,
-              n_decoder_layers=6,
-              n_heads=8,
-              dropout=0.1,
-              max_len=2048,
-              encoder_attention_type=tl.SelfAttention,
-              encoder_decoder_attention_type=tl.SelfAttention,
-              pos_type='fixed-base',
-              pos_axial_shape=(),
-              pos_d_axial_embs=None,
-              pos_start_from_zero_prob=1.0,
-              pos_max_offset_to_add=0,
-              ff_activation=tl.Relu,
-              ff_use_sru=0,
-              ff_chunk_size=0,
-              ff_dropout=None,
-              ff_sparsity=0,
-              loss_sparsity_type='mult',
-              loss_sparsity=0,
-              loss_d_lowrank=0,
-              loss_sparsity_prob=None,
-              attention_chunk_size=0,
-              n_layers_forget=0,
-              forget_dense=True,
-              n_decoder_attention_layers=2,
-              use_bfloat16=False,
-              reversible_encoder=False,
-              use_two_swaps_per_encoder_block=True,
-              center_layernorm=True,
-              half_before_layer=None,
-              double_after_layer=None,
-              mode='train'):
-  """Reversible transformer encoder-decoder model.
+def _InsertAxes12():
+  """Returns a layer that inserts two internal size-1 axes into an array."""
+  return tl.Fn('InsertAxes12',
+               lambda x: jnp.reshape(x, (x.shape[0], 1, 1, x.shape[1])))
 
-  This model expects an input pair: source, target.
 
-  At the moment, this model supports dot-product attention only. For the
-  attention types in the Reformer paper, see ReformerLM.
+def _RemoveAxes12():
+  """Returns a layer that removes two internal size-1 axes from an array."""
+  return tl.Fn('RemoveAxes12', lambda x: jnp.squeeze(x, (1, 2)))
 
-  Args:
-    input_vocab_size: int: vocab size of the source.
-    output_vocab_size: int (optional): vocab size of the target. If None, the
-      source and target are assumed to have the same vocab.
-    d_model: int:  depth of embedding
-    d_ff: int: depth of feed-forward layer
-    d_attention_key: int: depth of key vector for each attention head
-    d_attention_value: int: depth of value vector for each attention head
-    n_encoder_layers: int: number of encoder layers
-    n_decoder_layers: int: number of decoder layers
-    n_heads: int: number of attention heads
-    dropout: float: dropout rate (how much to drop out)
-    max_len: int: maximum symbol length for positional encoding
-    encoder_attention_type: class: attention class to use, such as SelfAttention
-    encoder_decoder_attention_type: class: attention class to use, such as
-      SelfAttention
-    pos_type: string, the type of positional embeddings to use.
-    pos_axial_shape: tuple of ints: input shape to use for the axial position
-      encoding. If unset, axial position encoding is disabled.
-    pos_d_axial_embs: tuple of ints: depth of position embedding for each axis.
-      Tuple length must match pos_axial_shape, and values must sum to d_model.
-    pos_start_from_zero_prob: how often to start from 0 during training,
-          (if 1.0, we always start from position 0, if less, we randomize).
-    pos_max_offset_to_add: maximum offset to add to positions during training
-        when randomizing; this offset plus input length must still be less than
-        max_len for all training examples.
-    ff_activation: the non-linearity in feed-forward layer
-    ff_use_sru: int; if > 0, we use this many SRU layers instead of feed-forward
-    ff_chunk_size: int; if > 0, chunk feed-forward into this-sized chunks
-    ff_dropout: float: (optional) separate dropout rate at feed-forward
-      nonlinearity. This is called relu_dropout in T2T.
-    ff_sparsity: int, if > 0 use sparse feed-forward block with this sparsity
-    loss_sparsity_type: str, type of sparsity to used in loss layer. See
-      SparseDenseWithOptions for options. None if no sparsity should be used.
-    loss_sparsity: int, the sparsity for loss layer (if used)
-    loss_d_lowrank: int, the dimensions for intermediate layer (if used)
-    loss_sparsity_prob: float, the probability for sparse version of loss to be
-      used. If None, only sparse version is used.
-    attention_chunk_size: int, if > 0 run attention chunked at this size
-    n_layers_forget: how often to have a forgetting block between layers
-    forget_dense: whether to use Dense or no-op (Serial) as a forget layer.
-    n_decoder_attention_layers: how many attention layers in a decoder block
-    use_bfloat16: whether to use bfloat16 for weights (default: False)
-    reversible_encoder: whether to be reversible through the encoder
-    use_two_swaps_per_encoder_block: whether to allow even number of swaps in
-      the encoder
-    center_layernorm: whether to use centering in LayerNorm (default) or if
-      to skip it, which is known as RMS normalization.
-    half_before_layer: int, half d_model and d_ff before that layer
-    double_after_layer: int, double d_model and d_ff after that layer
-    mode: str: 'train' or 'eval'
 
-  Returns:
-    A Reformer model as a layer that maps from a target, source pair to
-    activations over a vocab set.
-  """
-  # Set default dimensions for attention head key and value sizes.
-  if (d_model / 2) % n_heads != 0:
-    raise ValueError(f'n_heads ({n_heads}) must divide d_model/2 ({d_model/2})')
-  if d_attention_key is None:
-    d_attention_key = d_model // n_heads
-  if d_attention_value is None:
-    d_attention_value = d_model // n_heads
+def _AsTokenIDs():
+  """Returns a layer that makes mask values look like token ID ints."""
+  return tl.Fn('AsTokenIDs', lambda x: x.astype(jnp.int32))
 
-  # Set values of d_model, d_ff and d_qkv for the first stage.
-  d_model1, d_ff1 = d_model, d_ff
-  d_attention_key1, d_attention_value1 = d_attention_key, d_attention_value
-  if half_before_layer:
-    d_model1, d_ff1 = d_model / 2, d_ff / 2
-    d_attention_key1 = d_attention_key / 2
-    d_attention_value1 = d_attention_value / 2
 
-  # Set values of d_model, d_ff and d_qkv for the final stage.
-  d_model2, d_ff2 = d_model, d_ff
-  d_attention_key2, d_attention_value2 = d_attention_key, d_attention_value
-  if double_after_layer:
-    d_model2, d_ff2 = d_model * 2, d_ff * 2
-    d_attention_key2 = d_attention_key * 2
-    d_attention_value2 = d_attention_value * 2
-
-  # Vector embeddings.
-  in_encoder, out_encoder, output_vocab_size = (
-      ct.EmbeddingAndPositionalEncodings(
-          input_vocab_size,
-          d_model1,
-          mode,
-          dropout,
-          [-2],  # dropout_shared_axes
-          max_len,
-          output_vocab_size=output_vocab_size,
-          pos_type=pos_type,
-          pos_axial_shape=pos_axial_shape,
-          pos_d_axial_embs=pos_d_axial_embs,
-          pos_start_from_zero_prob=pos_start_from_zero_prob,
-          pos_max_offset_to_add=pos_max_offset_to_add,
-          use_bfloat16=use_bfloat16)
-  )
-
-  # pylint: disable=g-complex-comprehension
-  encoder_blocks = [
-      EncoderBlock(
-          d_model1, d_ff1, n_heads, encoder_attention_type,
-          dropout=dropout,
-          ff_activation=ff_activation,
-          ff_dropout=ff_dropout,
-          ff_use_sru=ff_use_sru,
-          ff_chunk_size=ff_chunk_size,
-          ff_sparsity=ff_sparsity,
-          attention_chunk_size=attention_chunk_size,
-          center_layernorm=center_layernorm,
-          use_bfloat16=use_bfloat16,
-          use_two_swaps_per_block=use_two_swaps_per_encoder_block,
-          mode=mode)
-      for _ in range(n_encoder_layers)]
-  # pylint: enable=g-complex-comprehension
-
-  encoder = [     # vec_e mask_e tok_e tok_d tok_d
-      tl.ReversibleSelect([0, 0]),     # vec_e1 vec_e2 mask_e tok_e tok_d tok_d
-      _ReversibleSerialForget(encoder_blocks, d_model1, n_layers_forget,
-                              forget_dense)
-  ]
-  if not reversible_encoder:
-    encoder += [
-        tl.Fn('XYAvg', lambda x, y: (x + y) / 2.0),
-        tl.Dense(d_model1, use_bfloat16=use_bfloat16),
-        tl.LayerNorm(),
-    ]
-  encoder = tl.Serial(encoder)
-  if mode == 'predict':
-    encoder = tl.Cache(encoder)
-
-  decoder_blocks = []
-
-  if isinstance(encoder_decoder_attention_type, (tuple, list)):
-    assert n_decoder_layers % len(encoder_decoder_attention_type) == 0
-  else:
-    encoder_decoder_attention_type = [encoder_decoder_attention_type]
-  for layer_idx in range(n_decoder_layers):
-    layer_attention_type = encoder_decoder_attention_type[
-        layer_idx % len(encoder_decoder_attention_type)]
-    # Grow d_model, d_ff, and d_qkv if requested.
-    d_m, d_f, d_k, d_v = d_model1, d_ff1, d_attention_key1, d_attention_value1
-    if half_before_layer and layer_idx >= half_before_layer:
-      d_m, d_f, d_k, d_v = d_model, d_ff, d_attention_key, d_attention_value
-    if double_after_layer and layer_idx > double_after_layer:
-      d_m, d_f, d_k, d_v = d_model2, d_ff2, d_attention_key2, d_attention_value2
-    decoder_block = DecoderBlock(
-        d_m, d_f, d_k, d_v, n_heads,
-        attention_type=layer_attention_type,
-        dropout=dropout,
-        ff_activation=ff_activation,
-        ff_dropout=ff_dropout,
-        ff_use_sru=ff_use_sru,
-        ff_chunk_size=ff_chunk_size,
-        ff_sparsity=ff_sparsity,
-        attention_chunk_size=attention_chunk_size,
-        n_attention_layers=n_decoder_attention_layers,
-        center_layernorm=center_layernorm,
-        use_bfloat16=use_bfloat16,
-        mode=mode)
-    decoder_blocks.append(decoder_block)
-    if half_before_layer and layer_idx == half_before_layer - 1:
-      decoder_blocks.append(tl.ReversibleConcatenatePair())
-    if double_after_layer and layer_idx == double_after_layer:
-      decoder_blocks.append(tl.ReversibleConcatenatePair())
-
-  dense_loss_layer = tl.SparseDenseWithOptions(
-      output_vocab_size,
-      d_input=d_model2,
-      sparsity_type=loss_sparsity_type,
-      sparsity=loss_sparsity,
-      d_lowrank=loss_d_lowrank,
-      prob_sparse=loss_sparsity_prob,
-      use_bfloat16=use_bfloat16,
-      mode=mode)
-
-  # Layers to merge encoder and decoder, see below for details.
-  if reversible_encoder:
-    encdec_layers = [
-        tl.ReversibleSelect([0, 1, 4, 2, 3]),  # vec_e vec_d mask_e tok_e tok_d
-        t2.ConcatWithPadding2(mode=mode),      # vec_ed vec_ed tok_e tok_d
-    ]
-  else:
-    encdec_layers = [
-        tl.ReversibleSelect([0, 3, 1, 2]),     # vec_e vec_d mask_e tok_e tok_d
-        t2.ConcatWithPadding(mode=mode),       # vec_ed tok_e tok_d
-        tl.ReversibleSelect([0, 0]),           # vec_ed vec_ed tok_e tok_d
-    ]
-
-  # Assemble and return the model.
-  return tl.Serial(
-      # Input: encoder_side_tokens, decoder_side_tokens
-      # Copy decoder tokens for use in loss.
-      tl.Select([0, 0, 0, 1, 1]),                # tok_e tok_e tok_e tok_d tok_d
-
-      # Embed in and out tokens; done together as weights may be shared.
-      tl.Parallel(in_encoder, [], [],            # vec_e tok_e tok_e vec_d tok_d
-                  [tl.ShiftRight(mode=mode), out_encoder]),
-
-      # Predict mode doesn't work with padding in encoder. Raising an exception
-      # in jitted function isn't possible, so the second next best thing is
-      # to convert every embedding to NaNs, so the user will not get subtly
-      # wrong results, but clearly wrong results.
-      (_ConvertToNaNsOnAnyZero() if mode == 'predict' else []),
-
-      tl.Parallel([], [tl.PaddingMask(),
-                       tl.Fn('Squeeze',
-                             lambda x: jnp.squeeze(x, (1, 2)), n_out=1)]),
-      #                                         # vec_e mask_e tok_e vec_d tok_d
-
-      # Encode.
-      encoder,                                  # vec_e mask_e tok_e vec_d tok_d
-
-      # Concat encoder and decoder, given encoder mask.
-      encdec_layers,
-
-      # Run decoder blocks.
-      _ReversibleSerialForget(decoder_blocks, d_model2, n_layers_forget,
-                              forget_dense),    # vec_ed1 vec_ed2 tok_e tok_d
-      tl.Fn('XYAvg',
-            lambda x, y: (x + y) / 2.0),           # vec_ed tok_e tok_d
-      tl.LayerNorm(),                              # vec_ed tok_e tok_d
-
-      # Separate out the encoder part from the concatenated vector.
-      tl.Select([0, 1, 2, 2]),                        # vec_ed tok_e tok_d tok_d
-      t2.StripFromConcatenateWithPadding(mode=mode),  # vec_d tok_d
-
-      # Map to output vocab.
-      dense_loss_layer,  # vec_d tok_d
-  )
+def _XYAvg():
+  """Returns a layer that computes the element-wise average of two arrays."""
+  return tl.Fn('XYAvg', lambda x, y: (x + y) / 2.0)
 
 
 def _ReversibleSerialForget(layers, d_model, n_layers, forget_dense=True):
@@ -862,10 +614,10 @@ def _ReversibleSerialForget(layers, d_model, n_layers, forget_dense=True):
 
   if forget_dense:
     forgetting_layer = tl.Serial(
-        tl.Fn('XYAvg', lambda x, y: (x + y) / 2.0),
+        _XYAvg(),
         tl.Dense(d_model),
         tl.Dup(),
-        )
+    )
   else:
     forgetting_layer = tl.Select([0, 1])
 
@@ -878,6 +630,6 @@ def _ReversibleSerialForget(layers, d_model, n_layers, forget_dense=True):
 
 def _ConvertToNaNsOnAnyZero():
   def _convert_to_nans(x, y):
-    # if all values in x are non-zeros, return x; otherwise return 0s
+    # if all values in y are non-zeros, return x; otherwise return 0s
     return jnp.where(jnp.all(y, keepdims=False), x, x/0.), y
   return tl.Fn('ConvertToNaNsOnAnyZero', _convert_to_nans, n_out=2)
