@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The Trax Authors.
+# Copyright 2021 The Trax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,6 +39,7 @@ from trax import optimizers as trax_opt
 from trax.data import inputs as trax_inputs
 from trax.fastmath import numpy as np
 from trax.fastmath import random as jax_random
+from trax.layers import base
 from trax.shapes import ShapeDtype
 from trax.supervised import history as trax_history
 from trax.supervised import lr_schedules as lr
@@ -62,15 +63,26 @@ OptState = collections.namedtuple('_OptState', [
 
 
 _DEFAULT_METRICS = {
-    'loss': tl.CrossEntropyLoss(),
-    'accuracy': tl.Accuracy(),
-    'sequence_accuracy': tl.SequenceAccuracy(),
-    'neg_log_perplexity': tl.Serial(tl.CrossEntropyLoss(), tl.Negate()),
-    'weights_per_batch_per_core': tl.SumOfWeights(),
+    'loss': tl.WeightedCategoryCrossEntropy(),
+    'accuracy': tl.WeightedCategoryAccuracy(),
+    'sequence_accuracy': tl.MaskedSequenceAccuracy(),
+    'neg_log_perplexity': tl.Serial(tl.WeightedCategoryCrossEntropy(),
+                                    tl.Negate()),
+    'weights_per_batch_per_core': tl.Serial(tl.Drop(), tl.Drop(), tl.Sum()),
 }
 
 
-class Trainer(object):
+NamedStream = collections.namedtuple(
+    'NamedStream', ['name', 'stream']
+)
+
+
+@gin.configurable
+def named_stream(name=gin.REQUIRED, stream=gin.REQUIRED):
+  return NamedStream(name=name, stream=stream)
+
+
+class Trainer:
   """Trax trainer.
 
   A trainer allows to make training steps, train for full epochs,
@@ -81,7 +93,9 @@ class Trainer(object):
                output_dir=None, random_seed=None, n_devices=None,
                checkpoints_at=None, should_save_checkpoints=True,
                should_write_summaries=True,
-               metrics=None, checkpoint_highest=None, checkpoint_lowest=None):
+               metrics=None, checkpoint_highest=None,
+               checkpoint_lowest=None,
+               init_checkpoint=None):
 
     self._is_chief, _, self._n_devices, rng = (
         training.init_host_and_devices(n_devices, random_seed))
@@ -104,6 +118,10 @@ class Trainer(object):
     # Setup the model.
     model_train = model(mode='train')
     model_predict_eval = model(mode='eval')
+    # Should work for fine-tuning of T5.
+    if init_checkpoint:
+      model_train.init_from_file(init_checkpoint, weights_only=True)
+      model_predict_eval.init_from_file(init_checkpoint, weights_only=True)
     self._model_with_loss = tl.Serial(model_train, loss_fn)
 
     # Setup state.
@@ -501,18 +519,20 @@ class Trainer(object):
       self._eval_sw = None
 
 
-@gin.configurable(blacklist=['output_dir'])
+@gin.configurable(denylist=['output_dir'])
 def train(output_dir,
           model=gin.REQUIRED,
-          loss_fn=tl.CrossEntropyLoss(),
+          loss_fn=tl.WeightedCategoryCrossEntropy(),
           inputs=trax_inputs.batcher,
           optimizer=trax_opt.Adafactor,
           lr_schedule_fn=lr.multifactor,
           trainer_class=Trainer,
           steps=1000,
           checkpoints_at=None,
+          permanent_checkpoints_at=None,
           eval_steps=10,
           eval_frequency=100,
+          permanent_checkpoint_frequency=None,
           random_seed=None,
           save_graphs=True,
           metrics=None,
@@ -520,7 +540,14 @@ def train(output_dir,
           checkpoint_lowest=None,
           use_loop=True,
           loss_chunk_size=0,
-          use_memory_efficient_trainer=False):
+          use_memory_efficient_trainer=False,
+          adasum=False,
+          init_checkpoint=None,
+          callbacks=None,
+          n_weights_shards=1,
+          additional_train_tasks=None,
+          additional_eval_tasks=None,
+          additional_eval_streams=None):
   """Train the model on the inputs.
 
   Args:
@@ -537,9 +564,13 @@ def train(output_dir,
     steps: int, total number of training steps.
     checkpoints_at: list of integers. Save a checkpoint for each training step
       in the list.
+    permanent_checkpoints_at: list of integers. Save a permanent checkpoint for
+      each training step in the list.
     eval_steps: int, num of steps per evaluation. If None or 0, eval disabled.
     eval_frequency: int, how often to run evaluation (every eval_frequency
       steps). If None or 0, eval disabled.
+    permanent_checkpoint_frequency: int, how often to save permanent checkpoints
+      (every permanent_checkpoint_frequency steps).
     random_seed: the random seed to use; time/os dependent if None (default).
     save_graphs: bool, if True, save computation graph to file.
     metrics: optionally override the default metrics dictionary.
@@ -548,23 +579,44 @@ def train(output_dir,
     use_loop: whether to use training.Loop instead of Trainer.
     loss_chunk_size: int, if > 0 chunk loss into these sizes to save memory.
     use_memory_efficient_trainer: whether to use memory-efficient trainer.
+    adasum: if True, use adaptive summation for multi-device gradients.
+    init_checkpoint: a checkpoint for fine tuning.
+    callbacks: a list of callbacks to call during training.
+    n_weights_shards: shard weights into this many devices.
+    additional_train_tasks: additional tasks which should be performed during
+      training.
+    additional_eval_tasks: additional tasks which should be performed during
+      evaluation.
+    additional_eval_streams: List[NamedStream], additional data streams that
+      should be used during evaluation. Can be provided independently of
+      additional_eval_tasks.
 
   Returns:
     trax.TrainerState or training.Loop if use_loop is True
   """
+  base.N_WEIGHTS_SHARDS = n_weights_shards
+  if (permanent_checkpoint_frequency is not None
+      and permanent_checkpoints_at is not None):
+    raise ValueError('Only one of ["permanent_checkpoint_frequency", '
+                     '"permanent_checkpoints_at"] should be set.')
   if use_loop:
-    n_devices = num_devices() or fastmath.device_count()
+    n_devices = num_devices() or fastmath.local_device_count()
 
     # Prepare the training task.
     # Inputs is either an Inputs instance or a function that returns it.
     if callable(inputs):  # If we pass a function, e.g., through gin, call it.
       inputs = inputs()
     opt = optimizer if use_memory_efficient_trainer else optimizer()
-    train_task = training.TrainTask(inputs.train_stream(n_devices),
-                                    loss_layer=loss_fn,
-                                    optimizer=opt,
-                                    lr_schedule=lr_schedule_fn(),
-                                    n_steps_per_checkpoint=eval_frequency)
+    train_task = training.TrainTask(
+        inputs.train_stream(n_devices),
+        loss_layer=loss_fn,
+        optimizer=opt,
+        lr_schedule=lr_schedule_fn(),
+        n_steps_per_checkpoint=eval_frequency,
+        n_steps_per_permanent_checkpoint=permanent_checkpoint_frequency)
+
+    if additional_train_tasks is None:
+      additional_train_tasks = []
 
     # Prepare the evaluation.
     metrics_dict = metrics if metrics is not None else _DEFAULT_METRICS
@@ -574,24 +626,56 @@ def train(output_dir,
                                   metric_names=names,
                                   n_eval_batches=eval_steps)
 
+    if additional_eval_tasks is None:
+      additional_eval_tasks = []
+
+    additional_eval_tasks_from_streams = []
+    if additional_eval_streams is not None:
+      for stream in additional_eval_streams:
+        additional_eval_tasks_from_streams.append(
+            training.EvalTask(stream.stream,
+                              metrics,
+                              metric_names=names,
+                              n_eval_batches=eval_steps,
+                              export_prefix=stream.name))
+
     # Prepare the training loop.
     checkpoint_at = None
     if checkpoints_at is not None:
       checkpoint_at = lambda step: step in checkpoints_at
+    permanent_checkpoint_at = None
+    if permanent_checkpoints_at is not None:
+      permanent_checkpoint_at = (lambda step: step in permanent_checkpoints_at)
+
+    # Setup the model.
+    model_train = model(mode='train')
+    model_predict_eval = model(mode='eval')
+    if init_checkpoint:
+      model_train.init_from_file(init_checkpoint, weights_only=True)
+      model_predict_eval.init_from_file(init_checkpoint, weights_only=True)
     loop = training.Loop(
-        model(mode='train'),
-        [train_task],
-        eval_model=model(mode='eval'),
-        eval_tasks=[eval_task],
+        model_train, [train_task] + additional_train_tasks,
+        eval_model=model_predict_eval,
+        eval_tasks=[eval_task] +
+        additional_eval_tasks + additional_eval_tasks_from_streams,
         output_dir=output_dir,
         checkpoint_at=checkpoint_at,
+        permanent_checkpoint_at=permanent_checkpoint_at,
         n_devices=n_devices,
         loss_chunk_size=loss_chunk_size,
         use_memory_efficient_trainer=use_memory_efficient_trainer,
-        random_seed=random_seed)
+        adasum=adasum,
+        random_seed=random_seed,
+        callbacks=callbacks,
+    )
+
+    steps_to_go = steps - loop.step
+    if steps_to_go <= 0:
+      log('Stop training, already reached the total training steps %d' % steps)
+      return loop
 
     # Train and return the loop.
-    loop.run(steps)
+    loop.run(steps_to_go)
     return loop
 
   n_devices = num_devices()
@@ -602,7 +686,8 @@ def train(output_dir,
                           checkpoints_at=checkpoints_at,
                           metrics=metrics,
                           checkpoint_lowest=checkpoint_lowest,
-                          checkpoint_highest=checkpoint_highest)
+                          checkpoint_highest=checkpoint_highest,
+                          init_checkpoint=init_checkpoint)
 
   epoch_steps = [steps]  # Only training if eval_frequency is 0 or None
   if eval_frequency and eval_steps > 0:
